package com.bcxin.tenant.open.domains.mappers;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.bcxin.tenant.open.domains.criterias.*;
import com.bcxin.tenant.open.domains.dtos.AttendanceCompanyGroupDTO;
import com.bcxin.tenant.open.domains.dtos.AttendanceCompanyGroupDetailDTO;
import com.bcxin.tenant.open.domains.dtos.AttendanceRepDTO;
import com.bcxin.tenant.open.domains.dtos.MyAttendanceRecordDTO;
import com.bcxin.tenant.open.domains.entities.TenantEmployeeAttendanceEntity;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Date;
import java.util.List;

@Component
public interface AttendanceMapper extends BaseMapper<TenantEmployeeAttendanceEntity> {

    List<AttendanceRepDTO> find(IPage<AttendanceRepDTO> page, @Param("criteria") AttendanceCriteria criteria);
    /**
     * <b> 根据人员ID和日期查询当日所有打卡记录 </b>
     * @param status 签到类型 非必填
     * @param tenantEmployeeId 雇员ID
     * @param createdDate 签到日期
     * @author ZXF
     * @create 2023/03/28 0028 19:58
     * @version
     * @注意事项 </b>
     */
    List<TenantEmployeeAttendanceEntity> getByDay(@Param("tenantEmployeeId") String tenantEmployeeId, @Param("createdDate") String createdDate);


    /**
     * 取出最新的签到信息
     * @param employeeId
     * @return
     */
    TenantEmployeeAttendanceEntity getMaxItemByEmployeeId(String employeeId);

    Collection<AttendanceCompanyGroupDTO> searchByAdvanceGroup(AttendanceAdvanceCriteria criteria);

    long getTotalCountByAdvanceGroup(AttendanceAdvanceCriteria criteria);

    Collection<AttendanceCompanyGroupDetailDTO> searchByAdvanceGroupDetail(AttendanceAdvanceDetailCriteria criteria);

    long getTotalCountByAdvanceGroupDetail(AttendanceAdvanceDetailCriteria criteria);

    void batchInsert(Collection<TenantEmployeeAttendanceEntity> data);

    void batchInsertForStation(Collection<TenantEmployeeAttendanceEntity> data);

    void batchInsertForJobStation(Collection<TenantEmployeeAttendanceEntity> data);

    Collection<TenantEmployeeAttendanceEntity> searchByRecently(MyRecentlyAttendanceCriteria criteria);

    long getTotalCountByRecently(MyRecentlyAttendanceCriteria criteria);

    int cleanRecentlyRecords(@Param("expiredTime")Date expiredTime);

    Collection<MyAttendanceRecordDTO> searchMyRecentlyRecords(MyRecentlyAttendanceCriteria criteria);
}
