package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.*;
import com.bcxin.tenant.open.domains.dtos.AttendanceCompanyGroupDTO;
import com.bcxin.tenant.open.domains.dtos.AttendanceCompanyGroupDetailDTO;
import com.bcxin.tenant.open.domains.dtos.AttendanceRepDTO;
import com.bcxin.tenant.open.domains.dtos.MyAttendanceRecordDTO;
import com.bcxin.tenant.open.domains.entities.DeviceCommunicatedLogEntity;
import com.bcxin.tenant.open.domains.entities.TenantEmployeeAttendanceEntity;
import com.bcxin.tenant.open.domains.mappers.AttendanceMapper;
import com.bcxin.tenant.open.domains.utils.DateUtils;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.enums.AttendanceSearchModel;
import com.bcxin.tenant.open.infrastructures.enums.RecordStatus;
import com.bcxin.tenant.open.infrastructures.enums.ReferenceType;
import com.bcxin.tenant.open.infrastructures.exceptions.NotSupportTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.UnAuthorizedTenantException;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.bcxin.tenant.open.domains.criterias.AttendanceCriteria.SearchModel.AdvanceSearch;

@Repository
public class AttendanceRepositoryImpl implements AttendanceRepository {

    private final AttendanceMapper attendanceMapper;

    public AttendanceRepositoryImpl(AttendanceMapper attendanceMapper) {
        this.attendanceMapper = attendanceMapper;
    }

    @Override
    public List<TenantEmployeeAttendanceEntity> getByDay(String tenantEmployeeId, String createdDate) {
        return this.attendanceMapper.getByDay(tenantEmployeeId, createdDate);
    }

    @Override
    public TenantEmployeeAttendanceEntity getById(Object id) {
        return null;
    }

    @Override
    public void insert(TenantEmployeeAttendanceEntity data) {
        this.attendanceMapper.insert(data);
    }

    @Override
    public void batchInsert(Collection<TenantEmployeeAttendanceEntity> data) {
        if (!CollectionUtils.isEmpty(data)) {
            this.attendanceMapper.batchInsert(data);
        }
    }

    @Override
    public void batchInsertForStation(Collection<TenantEmployeeAttendanceEntity> data) {
        if (!CollectionUtils.isEmpty(data)) {
            this.attendanceMapper.batchInsertForStation(data);
        }
    }

    @Override
    public void batchInsertForJobStation(Collection<TenantEmployeeAttendanceEntity> data) {
        if (!CollectionUtils.isEmpty(data)) {
            this.attendanceMapper.batchInsertForJobStation(data);
        }
    }

    @Override
    public void update(TenantEmployeeAttendanceEntity data) {

    }

    @Override
    public IPage<AttendanceRepDTO> find(AttendanceCriteria criteria) {
        int pageIndex = criteria.getPageIndex();
        int pageSize = criteria.getPageSize();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }
        IPage<AttendanceRepDTO> page = new Page<>(pageIndex, pageSize);
        List<AttendanceRepDTO> attendancePage = this.attendanceMapper.find(page, criteria);
        page.setRecords(attendancePage);
        return page;
    }

    @Override
    public TenantEmployeeAttendanceEntity getMaxItemByEmployeeId(String employeeId) {
        return this.attendanceMapper.getMaxItemByEmployeeId(employeeId);
    }

    @Override
    public Collection<TenantEmployeeAttendanceEntity> getEmployeeAttendances(String employeeId, Date beginDate, Date endDate) {
       LambdaQueryWrapper<TenantEmployeeAttendanceEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper = queryWrapper.eq(TenantEmployeeAttendanceEntity::getTenantEmployeeId,employeeId);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if(beginDate!=null) {
            queryWrapper = queryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, dateFormat.format(beginDate));
        }
        if(endDate!=null) {
            queryWrapper = queryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, dateFormat.format(endDate));
        }

        return this.attendanceMapper.selectList(queryWrapper);
    }

    @Override
    public EntityCollection<TenantEmployeeAttendanceEntity> search(AttendanceCriteria criteria) {
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        LambdaQueryWrapper<TenantEmployeeAttendanceEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        switch (criteria.getSearchModel()) {
            case AdvanceSearch -> {
                if (StringUtils.hasLength(criteria.getName())) {
                    lambdaQueryWrapper = lambdaQueryWrapper.likeRight(TenantEmployeeAttendanceEntity::getName, criteria.getName());
                }
                if (StringUtils.hasLength(criteria.getCompanyName())) {
                    lambdaQueryWrapper = lambdaQueryWrapper.likeRight(TenantEmployeeAttendanceEntity::getCompanyName, criteria.getCompanyName());
                }
                if (StringUtils.hasLength(criteria.getSecurityStationName())) {
                    lambdaQueryWrapper = lambdaQueryWrapper.likeRight(TenantEmployeeAttendanceEntity::getSecurityStationName, criteria.getSecurityStationName());
                }

                if (criteria.getCreatedStarDate() != null) {
                    lambdaQueryWrapper = lambdaQueryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, criteria.getCreatedStarDate());
                }
                if (criteria.getCreatedEndDate() != null) {
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(criteria.getCreatedEndDate());
                    calendar.add(1,Calendar.DATE);
                    Date endDate = calendar.getTime();
                    lambdaQueryWrapper = lambdaQueryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, endDate);
                }

                if (StringUtils.hasLength(criteria.getIdCardNo())) {
                    lambdaQueryWrapper = lambdaQueryWrapper.likeRight(TenantEmployeeAttendanceEntity::getIdCardNo, criteria.getIdCardNo());
                }
            }

            /**
             * 这些是针对保安员自己的APP页面
             */
            //此处获取的时间为LocalDateTime类型，去了纳秒
            case Today -> {
                return searchByToday(userModel, criteria);
            }
            case RecentWeek -> {
                lambdaQueryWrapper = lambdaQueryWrapper.eq(TenantEmployeeAttendanceEntity::getTenantEmployeeId,userModel.getEmployeeId());
                lambdaQueryWrapper = lambdaQueryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getWeekDate());
                lambdaQueryWrapper = lambdaQueryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getTomorrowDate());
            }
            case RecentMonth -> {
                lambdaQueryWrapper = lambdaQueryWrapper.eq(TenantEmployeeAttendanceEntity::getTenantEmployeeId,userModel.getEmployeeId());
                lambdaQueryWrapper = lambdaQueryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getMonthDate());
                lambdaQueryWrapper = lambdaQueryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getTomorrowDate());
            }
        }

        if (!CollectionUtils.isEmpty(criteria.getSuperviseDepartIds())) {
            lambdaQueryWrapper = lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getSuperviseDepartId, criteria.getSuperviseDepartIds());
        }

        if(!criteria.isHasManagedProjectIds())
        {
            if(!CollectionUtils.isEmpty(criteria.getOrganizationIds()) &&
                    !CollectionUtils.isEmpty(criteria.getOtherSpecialPermissionScopeIds())) {
                lambdaQueryWrapper.and((wrapper -> {
                    wrapper.in(TenantEmployeeAttendanceEntity::getOrganizationId, criteria.getOrganizationIds())
                            .or().in(TenantEmployeeAttendanceEntity::getReferenceNumber, criteria.getOtherSpecialPermissionScopeIds());
                }));
            }else if(!CollectionUtils.isEmpty(criteria.getOrganizationIds())){
                lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getOrganizationId, criteria.getOrganizationIds());
            }else if(!CollectionUtils.isEmpty(criteria.getOtherSpecialPermissionScopeIds())){
                lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getReferenceNumber, criteria.getOtherSpecialPermissionScopeIds());
            }
        }else {
            if (!CollectionUtils.isEmpty(criteria.getOrganizationIds()) &&
                    !CollectionUtils.isEmpty(criteria.getManagedProjectIds())) {
                lambdaQueryWrapper.and((wrapper -> {
                    wrapper.in(TenantEmployeeAttendanceEntity::getOrganizationId, criteria.getOrganizationIds())
                            .or().in(TenantEmployeeAttendanceEntity::getProjectId, criteria.getManagedProjectIds());
                }));
            } else if (!CollectionUtils.isEmpty(criteria.getOrganizationIds())) {
                lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getOrganizationId, criteria.getOrganizationIds());
            } else if (!CollectionUtils.isEmpty(criteria.getManagedProjectIds())) {
                lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getProjectId, criteria.getManagedProjectIds());
            }

            if (!CollectionUtils.isEmpty(criteria.getOtherSpecialPermissionScopeIds())) {
                lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getReferenceNumber, criteria.getOtherSpecialPermissionScopeIds());
            }
        }
        /*
        if (!criteria.isSupervise()) {
            lambdaQueryWrapper = lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getOrganizationId, criteria.getOrganizationIds());
        }
         */

        if(!CollectionUtils.isEmpty(criteria.getProjectIds())) {
            lambdaQueryWrapper = lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getProjectId, criteria.getProjectIds());
        }

        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(TenantEmployeeAttendanceEntity::getId);

        if (!criteria.ignorePage()) {
            IPage<TenantEmployeeAttendanceEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

            IPage<TenantEmployeeAttendanceEntity> attendanceEntityIPage =
                    this.attendanceMapper.selectPage(page, lambdaQueryWrapper);

            return EntityCollection.create(attendanceEntityIPage.getRecords(), criteria.getPageSize(), attendanceEntityIPage.getTotal());
        } else {

            /**
             * 按照分页取出数据
             */
            lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s, %s ", criteria.getSkip(), criteria.getPageSize()));
            Collection<TenantEmployeeAttendanceEntity> attendanceEntities
                    = this.attendanceMapper.selectList(lambdaQueryWrapper);

            return EntityCollection.createForNoTotalCount(attendanceEntities,criteria.getPageSize());
        }
    }

    @Override
    public EntityCollection<AttendanceCompanyGroupDTO> searchByAdvanceGroup(AttendanceAdvanceCriteria criteria) {
        Collection<AttendanceCompanyGroupDTO> result =
                this.attendanceMapper.searchByAdvanceGroup(criteria);
        if (!criteria.ignorePage()) {
            long totalCount = this.attendanceMapper.getTotalCountByAdvanceGroup(criteria);

            return EntityCollection.create(result, criteria.getPageSize(), totalCount);
        } else {
            return EntityCollection.createForNoTotalCount(result, criteria.getPageSize());
        }
    }

    @Override
    public EntityCollection<AttendanceCompanyGroupDetailDTO> searchByAdvanceGroupDetail(AttendanceAdvanceDetailCriteria criteria) {
        Collection<AttendanceCompanyGroupDetailDTO> result =
                this.attendanceMapper.searchByAdvanceGroupDetail(criteria);

        if (!criteria.ignorePage()) {
            long totalCount = this.attendanceMapper.getTotalCountByAdvanceGroupDetail(criteria);

            return EntityCollection.create(result, criteria.getPageSize(), totalCount);
        } else {
            return EntityCollection.createForNoTotalCount(result, criteria.getPageSize());
        }
    }

    @Override
    public int cleanRecentlyRecords() {
        Date now = new Date();
        Date expiredTime = DateUtils.addDay(now, -3);
        return this.attendanceMapper.cleanRecentlyRecords(expiredTime);
    }

    @Override
    public Collection<MyAttendanceRecordDTO> search(MyAttendanceCriteria criteria) {
        if (criteria.getSearchModel() == AttendanceSearchModel.Today) {
            return searchMyRecentlyRecords(criteria);
        }
        LambdaQueryWrapper<TenantEmployeeAttendanceEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();

        Collection<RecordStatus> recordStatuses = new ArrayList<>();
        recordStatuses.add(RecordStatus.SignIn);
        recordStatuses.add(RecordStatus.SignOut);

        lambdaQueryWrapper =lambdaQueryWrapper.in(TenantEmployeeAttendanceEntity::getRecordStatus, recordStatuses);

        lambdaQueryWrapper =lambdaQueryWrapper.eq(TenantEmployeeAttendanceEntity::getTenantEmployeeId, criteria.getEmployeeId());

        switch (criteria.getSearchModel()) {
            case RecentWeek -> {
                lambdaQueryWrapper = lambdaQueryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getWeekDate());
                lambdaQueryWrapper = lambdaQueryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getTomorrowDate());
            }
            case RecentMonth -> {
                lambdaQueryWrapper = lambdaQueryWrapper.ge(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getMonthDate());
                lambdaQueryWrapper = lambdaQueryWrapper.lt(TenantEmployeeAttendanceEntity::getCreatedTime, DateUtils.getTomorrowDate());
            }
            default -> {
                throw new NotSupportTenantException();
            }
        }

        /**
         * 按照分页取出数据
         */
        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s, %s ", criteria.getSkip(), criteria.getPageSize()));
        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(TenantEmployeeAttendanceEntity::getId);
        
        Collection<TenantEmployeeAttendanceEntity> attendanceEntities
                = this.attendanceMapper.selectList(lambdaQueryWrapper);

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return attendanceEntities.stream().map(ix -> MyAttendanceRecordDTO.create(
                dateFormat.format(ix.getCreatedTime()),
                ix.getRecordStatus(),
                ix.getName(),
                ix.getAddress()

        )).collect(Collectors.toList());
    }

    @Override
    public Collection<TenantEmployeeAttendanceEntity> getByTypeAndNumber(ReferenceType referenceType, String referenceNumber, int pageIndex, int pageSize) {
        LambdaQueryWrapper<TenantEmployeeAttendanceEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(TenantEmployeeAttendanceEntity::getReferenceType, referenceType.ordinal());
        lambdaQueryWrapper.eq(TenantEmployeeAttendanceEntity::getReferenceNumber, referenceNumber);
        if (pageSize < 1) {
            pageSize = 10;
        }

        if (pageIndex < 1) {
            pageIndex = 1;
        }

        int skip = (pageIndex - 1) * pageSize;

        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s, %s ", skip, pageSize));

        return this.attendanceMapper.selectList(lambdaQueryWrapper);
    }

    private Collection<MyAttendanceRecordDTO> searchMyRecentlyRecords(MyAttendanceCriteria criteria) {
        MyRecentlyAttendanceCriteria recentlyAttendanceCriteria = MyRecentlyAttendanceCriteria.create(
                criteria.getPageIndex(),
                criteria.getPageSize(),
                criteria.getEmployeeId(),
                DateUtils.getTodayDateValue()
        );

        Collection<MyAttendanceRecordDTO> records = this.attendanceMapper.searchMyRecentlyRecords(recentlyAttendanceCriteria);

        return records;
    }

    private EntityCollection<TenantEmployeeAttendanceEntity> searchByToday(TenantEmployeeContext.TenantUserModel userModel, AttendanceCriteria criteria) {
        MyRecentlyAttendanceCriteria recentlyAttendanceCriteria = MyRecentlyAttendanceCriteria.create(
                criteria.getPageIndex(),
                criteria.getPageSize(),
                userModel.getEmployeeId(),
                DateUtils.getTodayDateValue()
        );

        Collection<TenantEmployeeAttendanceEntity> entities = this.attendanceMapper.searchByRecently(recentlyAttendanceCriteria);
        long count = this.attendanceMapper.getTotalCountByRecently(recentlyAttendanceCriteria);

        return EntityCollection.create(entities, criteria.getPageSize(), count);
    }
}
