package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.DispatchLogDetailSearchCriteria;
import com.bcxin.tenant.open.domains.criterias.MyDispatchLogRecordCriteria;
import com.bcxin.tenant.open.domains.entities.DeviceCommunicatedLogDetailEntity;
import com.bcxin.tenant.open.domains.mappers.DeviceCommunicatedLogDetailMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

@Repository
public class DeviceCommunicatedLogDetailRepositoryImpl implements DeviceCommunicatedLogDetailRepository{
    private final DeviceCommunicatedLogDetailMapper deviceCommunicatedLogDetailMapper;

    public DeviceCommunicatedLogDetailRepositoryImpl(DeviceCommunicatedLogDetailMapper deviceCommunicatedLogDetailMapper) {
        this.deviceCommunicatedLogDetailMapper = deviceCommunicatedLogDetailMapper;
    }


    @Override
    public DeviceCommunicatedLogDetailEntity getById(Object id) {
        return this.deviceCommunicatedLogDetailMapper.selectById((Long) id);
    }

    @Override
    public void insert(DeviceCommunicatedLogDetailEntity data) {
        this.deviceCommunicatedLogDetailMapper.insert(data);
    }

    @Override
    public void update(DeviceCommunicatedLogDetailEntity data) {
        this.deviceCommunicatedLogDetailMapper.updateById(data);
    }

    @Override
    public EntityCollection<DeviceCommunicatedLogDetailEntity> search(DispatchLogDetailSearchCriteria criteria) {
        LambdaQueryWrapper<DeviceCommunicatedLogDetailEntity> queryWrapper = new LambdaQueryWrapper();

        boolean validQuery = false;
        if (StringUtils.hasLength(criteria.getLogId())) {
            queryWrapper = queryWrapper.eq(DeviceCommunicatedLogDetailEntity::getDeviceCommunicatedLogId, criteria.getLogId());
            validQuery = true;
        }

        if (StringUtils.hasLength(criteria.getCalledOrganizationId())) {
            queryWrapper = queryWrapper.eq(DeviceCommunicatedLogDetailEntity::getCalledDomainId, criteria.getCalledOrganizationId());
            validQuery = true;
        }

        if (criteria.getReferenceType() != null) {
            queryWrapper = queryWrapper.eq(DeviceCommunicatedLogDetailEntity::getReferenceType, criteria.getReferenceType().ordinal());
            validQuery = true;
        }

        if (StringUtils.hasLength(criteria.getReferenceNumber())) {
            queryWrapper = queryWrapper.eq(DeviceCommunicatedLogDetailEntity::getReferenceNumber, criteria.getReferenceNumber());
            validQuery = true;
        }

        if (!validQuery) {
            return EntityCollection.EMPTY();
        }

        if (criteria.isIgnoreCount()) {
            queryWrapper = queryWrapper.orderByAsc(DeviceCommunicatedLogDetailEntity::getCreatedTime);
            Collection<DeviceCommunicatedLogDetailEntity> data =
                    this.deviceCommunicatedLogDetailMapper.selectList(queryWrapper);

            return EntityCollection.create(data, criteria.getPageSize(), (long)data.size());
        } else {
            int pageIndex = criteria.getPageIndex();
            if (pageIndex <= 0) {
                pageIndex = 1;
            }

            IPage<DeviceCommunicatedLogDetailEntity> page = new Page<>(pageIndex, criteria.getPageSize());
            queryWrapper = queryWrapper.orderByAsc(DeviceCommunicatedLogDetailEntity::getCreatedTime);
            IPage<DeviceCommunicatedLogDetailEntity> data =
                    this.deviceCommunicatedLogDetailMapper.selectPage(page, queryWrapper);

            return EntityCollection.create(data.getRecords(), criteria.getPageSize(), data.getTotal());
        }
    }

    @Override
    public void batchInsert(Collection<DeviceCommunicatedLogDetailEntity> logs) {
        logs.forEach(ix -> {
            this.deviceCommunicatedLogDetailMapper.insert(ix);
        });
    }

    @Override
    public DeviceCommunicatedLogDetailEntity getByNoPkId(String id) {
        LambdaQueryWrapper<DeviceCommunicatedLogDetailEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceCommunicatedLogDetailEntity::getId, id);

        return this.deviceCommunicatedLogDetailMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<DeviceCommunicatedLogDetailEntity> search(MyDispatchLogRecordCriteria criteria) {
        LambdaQueryWrapper<DeviceCommunicatedLogDetailEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceCommunicatedLogDetailEntity::getCalledUserId, criteria.getEmployeeId());
        if (criteria.getBeginDate() != null) {
            lambdaQueryWrapper = lambdaQueryWrapper.ge(DeviceCommunicatedLogDetailEntity::getCreatedTime, criteria.getBeginDate());
        }

        if (criteria.getEndDate() != null) {
            Date endDate = criteria.getEndDate();
            Calendar endDateCalender = Calendar.getInstance();
            endDateCalender.setTime(endDate);
            endDateCalender.add(Calendar.DAY_OF_MONTH, 1);
            endDate = endDateCalender.getTime();

            lambdaQueryWrapper = lambdaQueryWrapper.lt(DeviceCommunicatedLogDetailEntity::getCreatedTime, endDate);
        }

        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s,%s", criteria.getSkip(), criteria.getPageSize()));

        return this.deviceCommunicatedLogDetailMapper.selectList(lambdaQueryWrapper);
    }
}
