package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.DispatchLogSearchCriteria;
import com.bcxin.tenant.open.domains.criterias.SponsorDispatchLogSearchCriteria;
import com.bcxin.tenant.open.domains.dtos.SponsorDispatchLogDTO;
import com.bcxin.tenant.open.domains.entities.DeviceCommunicatedLogDetailEntity;
import com.bcxin.tenant.open.domains.entities.DeviceCommunicatedLogEntity;
import com.bcxin.tenant.open.domains.mappers.DeviceCommunicatedLogMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.utils.DateUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Timestamp;
import java.util.Collection;
import java.util.stream.Collectors;

@Repository
public class DeviceCommunicatedLogRepositoryImpl implements DeviceCommunicatedLogRepository {
    private final DeviceCommunicatedLogMapper deviceCommunicatedLogMapper;

    public DeviceCommunicatedLogRepositoryImpl(DeviceCommunicatedLogMapper deviceCommunicatedLogMapper) {
        this.deviceCommunicatedLogMapper = deviceCommunicatedLogMapper;
    }

    @Override
    public DeviceCommunicatedLogEntity getById(Object id) {
        return this.deviceCommunicatedLogMapper.selectById((Long) id);
    }

    @Override
    public void insert(DeviceCommunicatedLogEntity data) {
        this.deviceCommunicatedLogMapper.insert(data);
    }

    @Override
    public void update(DeviceCommunicatedLogEntity data) {
        this.deviceCommunicatedLogMapper.updateById(data);
    }

    @Override
    public EntityCollection<DeviceCommunicatedLogEntity> search(DispatchLogSearchCriteria criteria) {
        LambdaQueryWrapper<DeviceCommunicatedLogEntity> queryWrapper = new LambdaQueryWrapper<>();
        /**
         * 调度发企业
         */
        if (StringUtils.hasLength(criteria.getCallerName())) {
            queryWrapper = queryWrapper.likeLeft(DeviceCommunicatedLogEntity::getCallerCompanyName, criteria.getCallerName().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getCalledCompanyName())) {
            queryWrapper = queryWrapper.likeLeft(DeviceCommunicatedLogEntity::getCalledCompanyName, criteria.getCalledCompanyName().trim().concat("%"));
        }

        if (!CollectionUtils.isEmpty(criteria.getCallerDeviceNos())) {
            queryWrapper = queryWrapper.in(DeviceCommunicatedLogEntity::getCallerDeviceNo, criteria.getCallerDeviceNos());
        }

        if (!CollectionUtils.isEmpty(criteria.getSuperviseDepartIds())) {
            queryWrapper = queryWrapper.in(DeviceCommunicatedLogEntity::getDomainId, criteria.getSuperviseDepartIds());
        }

        if (criteria.getDispatchedDateFrom() != null) {
            queryWrapper = queryWrapper.ge(DeviceCommunicatedLogEntity::getBeginTime, criteria.getDispatchedDateFrom());
        }

        if (criteria.getDeskTypes() != null) {
            queryWrapper = queryWrapper.in(DeviceCommunicatedLogEntity::getDeskType,
                    criteria.getDeskTypes().stream().map(ii -> ii.ordinal()).collect(Collectors.toSet())
            );
        }

        if (criteria.getDispatchedDateTo() != null) {
            queryWrapper.and((wrapper -> {
                wrapper.le(DeviceCommunicatedLogEntity::getEndTime, criteria.getDispatchedDateTo())
                        .or().isNull(DeviceCommunicatedLogEntity::getEndTime);
            }));
        }

        if (StringUtils.hasLength(criteria.getProjectName())) {
            queryWrapper = queryWrapper.likeLeft(DeviceCommunicatedLogEntity::getProjectName, criteria.getProjectName().trim().concat("%"));
        }

        int pageIndex = criteria.getPageIndex();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }
        queryWrapper = queryWrapper.orderByDesc(DeviceCommunicatedLogEntity::getCreatedTime);

        if (!criteria.ignorePage()) {
            IPage<DeviceCommunicatedLogEntity> page = new Page<>(pageIndex, criteria.getPageSize());

            IPage<DeviceCommunicatedLogEntity> deviceCommunicatedLogs =
                    this.deviceCommunicatedLogMapper.selectPage(page, queryWrapper);

            return EntityCollection.create(
                    deviceCommunicatedLogs.getRecords(),
                    criteria.getPageSize(),
                    deviceCommunicatedLogs.getTotal()
            );
        } else {

            /**
             * 按照分页取出数据
             */
            queryWrapper = queryWrapper.last(String.format("limit %s, %s ", criteria.getSkip(), criteria.getPageSize()));
            Collection<DeviceCommunicatedLogEntity> deviceCommunicatedLogs =
                    this.deviceCommunicatedLogMapper.selectList(queryWrapper);

            return EntityCollection.createForNoTotalCount(deviceCommunicatedLogs,criteria.getPageSize());
        }
    }

    @Override
    public void batchInsert(Collection<DeviceCommunicatedLogEntity> logs) {
        for (DeviceCommunicatedLogEntity ix : logs) {
            this.deviceCommunicatedLogMapper.insert(ix);
        }
    }

    @Override
    public DeviceCommunicatedLogEntity getByNoPkId(String id) {
        LambdaQueryWrapper<DeviceCommunicatedLogEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(DeviceCommunicatedLogEntity::getId, id);

        return this.deviceCommunicatedLogMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<SponsorDispatchLogDTO> search(SponsorDispatchLogSearchCriteria criteria) {
        return this.deviceCommunicatedLogMapper.searchSponsorDispatch(criteria);
    }
}