package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.bcxin.tenant.open.domains.entities.MonthlyBillEntity;
import com.bcxin.tenant.open.domains.mappers.MonthlyBillMapper;
import com.bcxin.tenant.open.infrastructures.enums.BillPaymentStatus;
import org.springframework.stereotype.Repository;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.Collection;
import java.util.List;


@Repository
public class MonthlyBillRepositoryImpl implements MonthlyBillRepository {

    private final MonthlyBillMapper billMapper;

    public MonthlyBillRepositoryImpl(MonthlyBillMapper billMapper) {
        this.billMapper = billMapper;
    }

    @Override
    public MonthlyBillEntity getById(Object id) {
        return null;
    }

    @Override
    public void insert(MonthlyBillEntity data) {

    }

    @Override
    public void update(MonthlyBillEntity data) {
        LambdaUpdateWrapper<MonthlyBillEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(MonthlyBillEntity::getBillId,data.getBillId());
        billMapper.update(data,updateWrapper);
    }

    @Override
    public void generateBills(List<MonthlyBillEntity> billEntityList) {
        billMapper.generateBills(billEntityList);
    }

    @Override
    public void updatePayment(String billId, Integer paidAmount) {
        LambdaUpdateWrapper<MonthlyBillEntity> updateWrapper = new LambdaUpdateWrapper();
        updateWrapper.set(MonthlyBillEntity::getPaidAmount,paidAmount)
                .set(MonthlyBillEntity::getPaidTime, Timestamp.from(Instant.now()))
                .eq(MonthlyBillEntity::getBillId,billId);
        billMapper.update(null, updateWrapper);
    }

    @Override
    public void insertEmptyBills(List<MonthlyBillEntity> billEntityList) {
        if(billEntityList.isEmpty()) return;
        billMapper.insertEmptyBills(billEntityList);
    }

    @Override
    public List<MonthlyBillEntity> getByBillId(List<String> billIds) {
        LambdaQueryWrapper<MonthlyBillEntity> queryWrapper = new LambdaQueryWrapper();
        queryWrapper.in(MonthlyBillEntity::getBillId, billIds);
        return billMapper.selectList(queryWrapper);
    }

    @Override
    public List<MonthlyBillEntity> getRequiredPaymentBillsByMonth(String month) {
        LambdaQueryWrapper<MonthlyBillEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MonthlyBillEntity::getMonth, month);
        queryWrapper.eq(MonthlyBillEntity::getIsReady, 1);
        queryWrapper.in(MonthlyBillEntity::getBillPaymentStatus, BillPaymentStatus.Unpaid.ordinal(),BillPaymentStatus.PartialPaid.ordinal());
        return billMapper.selectList(queryWrapper);
    }

    @Override
    public List<MonthlyBillEntity> getRequiredPaymentBills() {
        LambdaQueryWrapper<MonthlyBillEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MonthlyBillEntity::getIsReady, 1);
        queryWrapper.in(MonthlyBillEntity::getBillPaymentStatus, BillPaymentStatus.Unpaid.ordinal(),BillPaymentStatus.PartialPaid.ordinal());
        return billMapper.selectList(queryWrapper);
    }

    @Override
    public List<MonthlyBillEntity> getRequiredPaymentBillsByOrgs(Collection<String> orgIds) {
        LambdaQueryWrapper<MonthlyBillEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(MonthlyBillEntity::getOrgId, orgIds);
        queryWrapper.eq(MonthlyBillEntity::getIsReady, 1);
        queryWrapper.in(MonthlyBillEntity::getBillPaymentStatus, BillPaymentStatus.Unpaid.ordinal(),BillPaymentStatus.PartialPaid.ordinal());
        return billMapper.selectList(queryWrapper);
    }

    @Override
    public void updatePaidAmount(List<MonthlyBillEntity> billEntityList) {
        if(billEntityList.isEmpty()){
            return;
        }
        this.billMapper.updatePaidAmount(billEntityList);
    }
}
