package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.CompanyAdvanceCriteria;
import com.bcxin.tenant.open.domains.dtos.CompanyAdvanceDTO;
import com.bcxin.tenant.open.domains.entities.RdEmployeeEntity;
import com.bcxin.tenant.open.domains.mappers.RdCompanyMapper;
import com.bcxin.tenant.open.domains.criterias.RdCompanyCriteria;
import com.bcxin.tenant.open.domains.entities.RdCompanyEntity;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Repository
public class RdCompanyRepositoryImpl implements RdCompanyRepository {

    private final RdCompanyMapper rdCompanyMapper;

    public RdCompanyRepositoryImpl(RdCompanyMapper rdCompanyMapper) {
        this.rdCompanyMapper = rdCompanyMapper;
    }

    @Override
    public RdCompanyEntity getById(Object id) {
        return this.rdCompanyMapper.selectById((Long) id);
    }

    @Override
    public void insert(RdCompanyEntity data) {
        this.rdCompanyMapper.insert(data);
    }

    @Override
    public void update(RdCompanyEntity data) {
        this.rdCompanyMapper.updateById(data);
    }

    @Override
    public Collection<RdCompanyEntity> find(RdCompanyCriteria criteria) {
        return null;
    }

    @Override
    public Collection<RdCompanyEntity> getByPage(Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdCompanyEntity> wrapper = new LambdaQueryWrapper();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }

        if(!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdCompanyEntity::getId, ids);
        }

        IPage<RdCompanyEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdCompanyEntity> companyEntityPage =
                this.rdCompanyMapper.selectPage(page, wrapper);

        return companyEntityPage.getRecords();
    }

    @Override
    public RdCompanyEntity getByNoPkId(String id) {
        LambdaQueryWrapper<RdCompanyEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(RdCompanyEntity::getId, id);

        return this.rdCompanyMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<String> getCompanyIds(int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdCompanyEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.select(RdCompanyEntity::getId);
        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s,%s", (pageIndex - 1) * pageSize, pageSize));
        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(RdCompanyEntity::getPkId);

        List<Map<String, Object>> maps = this.rdCompanyMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii -> String.valueOf(ii.get("id"))).collect(Collectors.toList());
    }

    @Override
    public EntityCollection<CompanyAdvanceDTO> findByStations(CompanyAdvanceCriteria criteria) {
        Collection<CompanyAdvanceDTO> data =
                this.rdCompanyMapper.find(criteria);

        long totalCount = this.rdCompanyMapper.count(criteria);

        return EntityCollection.create(data, criteria.getPageSize(), totalCount);
    }

    @Override
    public List<String> selectAllCompanyIds() {
        LambdaQueryWrapper<RdCompanyEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.select(RdCompanyEntity::getId);
        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(RdCompanyEntity::getPkId);

        List<Map<String, Object>> maps = this.rdCompanyMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii -> String.valueOf(ii.get("id"))).collect(Collectors.toList());
    }

}
