package com.bcxin.tenant.open.domains.repositories;

import com.bcxin.tenant.open.document.domains.documents.RdEmployeeDocument;
import com.bcxin.tenant.open.document.domains.documents.RdEmployeeDocument$;
import com.bcxin.tenant.open.document.domains.documents.RdSecurityStationDocument;
import com.bcxin.tenant.open.document.domains.documents.RdSecurityStationDocument$;
import com.bcxin.tenant.open.document.domains.dtos.RdCompanyCountOfSecurityManDTO;
import com.bcxin.tenant.open.document.domains.dtos.RdCompanyCountOfStationDTO;
import com.bcxin.tenant.open.domains.criterias.ProjectRoadmapCriteria;
import com.bcxin.tenant.open.domains.dtos.ProjectRoadmapDTO;
import com.bcxin.tenant.open.domains.dtos.SecurityStationSuperviseDTO;
import com.bcxin.tenant.open.domains.mappers.RdDeviceDeskServiceScopeViewMapper;
import com.bcxin.tenant.open.domains.mappers.RdProjectRoadmapMapper;
import com.bcxin.tenant.open.domains.mappers.RdSecurityStationPersonMapper;
import com.bcxin.tenant.open.domains.mappers.RdTenantUserAccountViewMapper;
import com.bcxin.tenant.open.domains.readers.RdCompositedReader;
import com.bcxin.tenant.open.domains.views.RdDeviceDeskServiceScopeView;
import com.bcxin.tenant.open.domains.views.RdTenantUserAccountView;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.redis.om.spring.search.stream.EntityStream;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.stream.Collectors;

@Repository
public class RdCompositedReaderImpl implements RdCompositedReader {
    private final RdTenantUserAccountViewMapper tenantUserAccountViewMapper;
    private final RdDeviceDeskServiceScopeViewMapper deviceDeskServiceScopeViewMapper;
    private final RdSecurityStationPersonMapper securityStationPersonMapper;
    private final EntityStream entityStream;

    private final RdProjectRoadmapMapper projectRoadmapMapper;

    public RdCompositedReaderImpl(RdTenantUserAccountViewMapper tenantUserAccountViewMapper,
                                  RdDeviceDeskServiceScopeViewMapper deviceDeskServiceScopeViewMapper,
                                  RdSecurityStationPersonMapper securityStationPersonMapper,
                                  EntityStream entityStream, RdProjectRoadmapMapper projectRoadmapMapper) {
        this.tenantUserAccountViewMapper = tenantUserAccountViewMapper;
        this.deviceDeskServiceScopeViewMapper = deviceDeskServiceScopeViewMapper;
        this.securityStationPersonMapper = securityStationPersonMapper;
        this.entityStream = entityStream;
        this.projectRoadmapMapper = projectRoadmapMapper;
    }

    @Override
    public Collection<RdDeviceDeskServiceScopeView> getByEmployeeIds(Collection<String> employeeIds) {
        return this.deviceDeskServiceScopeViewMapper.getByBoundEmployeeIds(employeeIds);
    }

    @Override
    public RdTenantUserAccountView getUserById(String tenantUserId) {
        return this.tenantUserAccountViewMapper.getUserById(tenantUserId);
    }

    @Override
    public Collection<SecurityStationSuperviseDTO> getValidSecurityStationSupervisesByEmployeeIds(Collection<String> employeeIds) {
        return this.securityStationPersonMapper.getAllValidSecurityStationsByEmployeeIds(employeeIds);
    }

    @Override
    public Long getCountOfSecurityManByOrganizationId(String organizationId) {
        Long countOfSecurityMan =
                entityStream.of(RdEmployeeDocument.class)
                        .filter(RdEmployeeDocument$.ORGANIZATION_ID.eq(organizationId))
                        .count();
        return countOfSecurityMan;
    }

    @Override
    public Long getCountOfSecurityManByStationId(String stationId) {
        Long countOfSecurityMan =
                entityStream.of(RdEmployeeDocument.class)
                        .filter(RdEmployeeDocument$.SECURITY_STATION_ID.eq(stationId))
                        .count();
        return countOfSecurityMan;
    }

    @Override
    public Long getCountOfActivityStationByOrganizationId(String organizationId) {
        Long countOfActivityStation =
                entityStream.of(RdSecurityStationDocument.class)
                        .filter(RdSecurityStationDocument$.ORGANIZATION_ID.eq(organizationId))
                        .filter(RdSecurityStationDocument$.IS_ACTIVATED.eq(true))
                        .count();

        return countOfActivityStation;
    }

    @Override
    public Collection<RdCompanyCountOfSecurityManDTO> getCountOfSecurityManByOrganizationIds(Collection<String> organizationIds) {
        String[] organizationIdsArray =
                organizationIds.toArray(new String[organizationIds.size()]);

        return organizationIds.stream().map(ii -> {
            Long count =
                    this.entityStream.of(RdEmployeeDocument.class)
                            .filter(RdEmployeeDocument$.ORGANIZATION_ID.in(organizationIdsArray))
                            .count();
            RdCompanyCountOfSecurityManDTO dto = new RdCompanyCountOfSecurityManDTO();
            dto.setOrganizationId(ii);
            dto.setTotal(count);

            return dto;
        }).collect(Collectors.toList());
    }

    @Override
    public Collection<RdCompanyCountOfStationDTO> getCountOfStationByOrganizationIds(Collection<String> organizationIds) {
        String[] organizationIdsArray =
                organizationIds.toArray(new String[organizationIds.size()]);

        return
                organizationIds.stream().map(ii -> {
                    RdCompanyCountOfStationDTO dto = new RdCompanyCountOfStationDTO();
                    dto.setOrganizationId(ii);
                    Long count =
                            this.entityStream.of(RdSecurityStationDocument.class)
                                    .filter(RdSecurityStationDocument$.ORGANIZATION_ID.in(organizationIdsArray))
                                    .filter(RdSecurityStationDocument$.IS_ACTIVATED.eq(true)).count();
                    dto.setTotal(count);

                    return dto;
                }).collect(Collectors.toList());
    }

    @Override
    public EntityCollection<ProjectRoadmapDTO> search(ProjectRoadmapCriteria criteria) {
        Collection<ProjectRoadmapDTO> roadmaps = this.projectRoadmapMapper.search(criteria);
        long totalCount = this.projectRoadmapMapper.getTotalCount(criteria);

        return EntityCollection.create(roadmaps, criteria.getPageSize(), totalCount);
    }

    @Override
    public Collection<RdDeviceDeskServiceScopeView> getSuperiorDeskByBoundEmployeeIds(Collection<String> employeeIds) {
        return this.deviceDeskServiceScopeViewMapper.getSuperiorDeskByBoundEmployeeIds(employeeIds);
    }
}
