package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.dtos.DeviceBatchUpdateDTO;
import com.bcxin.tenant.open.domains.entities.RdDeviceEntity;
import com.bcxin.tenant.open.domains.mappers.RdDeviceMapper;
import org.springframework.stereotype.Repository;
import org.springframework.ui.freemarker.FreeMarkerTemplateUtils;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.Collections;
import java.util.Optional;
import java.util.stream.Collectors;

@Repository
public class RdDeviceRepositoryImpl  implements RdDeviceRepository {
    private final RdDeviceMapper deviceMapper;

    public RdDeviceRepositoryImpl(RdDeviceMapper deviceMapper) {
        this.deviceMapper = deviceMapper;
    }

    @Override
    public RdDeviceEntity getById(Object id) {
        return this.deviceMapper.selectById((Long) id);
    }

    @Override
    public void insert(RdDeviceEntity data) {
        this.deviceMapper.insert(data);
    }

    @Override
    public void update(RdDeviceEntity data) {
        this.deviceMapper.updateById(data);
    }

    @Override
    public Collection<RdDeviceEntity> getByPage(Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdDeviceEntity> wrapper = new LambdaQueryWrapper();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdDeviceEntity::getId, ids);
        }

        IPage<RdDeviceEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdDeviceEntity> deviceEntityIPage =
                this.deviceMapper.selectPage(page, wrapper);

        return deviceEntityIPage.getRecords();
    }

    @Override
    public Collection<RdDeviceEntity> getNoReadyDevices(Collection<String> ids) {
        if(CollectionUtils.isEmpty(ids)) {
            return Collections.EMPTY_LIST;
        }

        LambdaQueryWrapper<RdDeviceEntity> wrapper = new LambdaQueryWrapper();
        wrapper = wrapper.in(RdDeviceEntity::getId, ids);

        return this.deviceMapper.selectList(wrapper);
    }

    @Override
    public Collection<String>  batchUpdate(DeviceBatchUpdateDTO data) {
        if (data == null || CollectionUtils.isEmpty(data.getItems())) {
            return Collections.EMPTY_LIST;
        }

        Collection<String> deviceNos =
                data.getItems().stream().map(ii -> ii.getDeviceNo())
                        .collect(Collectors.toSet());
        LambdaQueryWrapper<RdDeviceEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper =
                lambdaQueryWrapper.in(RdDeviceEntity::getDeviceNo, deviceNos);

        Collection<RdDeviceEntity> devices =
                this.deviceMapper.selectList(lambdaQueryWrapper);

        for (RdDeviceEntity device : devices) {
            Optional<String> deviceUidOptional = data.getItems().stream()
                    .filter(ii -> ii.getDeviceNo().equals(device.getDeviceNo()))
                    .map(ii -> ii.getDeviceUid()).findFirst();
            if (deviceUidOptional.isPresent()) {
                device.setDeviceUid(deviceUidOptional.get());
            }

            this.deviceMapper.updateById(device);
        }

        return devices.stream().map(ii -> ii.getId())
                .collect(Collectors.toSet());
    }
}