package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.entities.PoliceIncidentsEntity;
import com.bcxin.tenant.open.domains.entities.RdDispatchDataScopeEntity;
import com.bcxin.tenant.open.domains.entities.RdEmployeeEntity;
import com.bcxin.tenant.open.domains.mappers.RdDispatchDataScopeMapper;
import com.bcxin.tenant.open.infrastructures.enums.DispatchDataScopeType;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.stream.Collectors;

@Repository
public class RdDispatchDataScopeRepositoryImpl implements RdDispatchDataScopeRepository {

    private final RdDispatchDataScopeMapper dispatchDataScopeMapper;

    public RdDispatchDataScopeRepositoryImpl(RdDispatchDataScopeMapper dispatchDataScopeMapper) {
        this.dispatchDataScopeMapper = dispatchDataScopeMapper;
    }

    @Override
    public RdDispatchDataScopeEntity getById(Object id) {

        return this.dispatchDataScopeMapper.selectById((String) id);
    }

    @Override
    public void insert(RdDispatchDataScopeEntity data) {
        this.dispatchDataScopeMapper.insert(data);
    }

    @Override
    public void update(RdDispatchDataScopeEntity data) {
        this.dispatchDataScopeMapper.updateById(data);
    }

    @Override
    public Collection<RdDispatchDataScopeEntity> getByPage(DispatchDataScopeType dispatchDataScopeType, Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdDispatchDataScopeEntity> wrapper = new LambdaQueryWrapper();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }

        if(dispatchDataScopeType!=null) {
            wrapper = wrapper.eq(RdDispatchDataScopeEntity::getScopeType, dispatchDataScopeType.ordinal());
        }

        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.and(ix ->
                    ix.in(RdDispatchDataScopeEntity::getId, ids).or()
                            .in(RdDispatchDataScopeEntity::getId,
                                    ids.stream().map(ii -> String.format("%s_%s", dispatchDataScopeType.ordinal(), ids))
                                            .collect(Collectors.toList())
                            )
            );
        }

        IPage<RdDispatchDataScopeEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdDispatchDataScopeEntity> dataScopeEntityIPage =
                this.dispatchDataScopeMapper.selectPage(page, wrapper);

        return dataScopeEntityIPage.getRecords();
    }
}
