package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.dtos.UpdateEmployeeLocationDTO;
import com.bcxin.tenant.open.domains.entities.RdEmployeeEntity;
import com.bcxin.tenant.open.domains.mappers.RdEmployeeMapper;
import com.bcxin.tenant.open.domains.mappers.RdTenantUserAccountViewMapper;
import com.bcxin.tenant.open.domains.mappers.TenantUserViewMapper;
import com.bcxin.tenant.open.infrastructures.constants.BusinessConstants;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class RdEmployeeRepositoryImpl implements RdEmployeeRepository {

    private final RdEmployeeMapper employeeMapper;
    private final SqlSessionFactory sqlSessionFactory;
    private final RdTenantUserAccountViewMapper tenantUserViewMapper;

    public RdEmployeeRepositoryImpl(RdEmployeeMapper employeeMapper,
                                    SqlSessionFactory sqlSessionFactory,
                                    RdTenantUserAccountViewMapper tenantUserViewMapper) {
        this.employeeMapper = employeeMapper;
        this.sqlSessionFactory = sqlSessionFactory;
        this.tenantUserViewMapper = tenantUserViewMapper;
    }

    @Override
    public RdEmployeeEntity getById(Object id) {
        return employeeMapper.selectById((Long) id);
    }

    @Override
    public void insert(RdEmployeeEntity data) {
        this.employeeMapper.insert(data);
    }

    @Override
    public void update(RdEmployeeEntity data) {
        this.employeeMapper.updateById(data);
    }

    @Override
    public Collection<RdEmployeeEntity> getByPage(Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdEmployeeEntity> wrapper = new LambdaQueryWrapper();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdEmployeeEntity::getId, ids);
        }

        IPage<RdEmployeeEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdEmployeeEntity> employeeEntityPage =
                this.employeeMapper.selectPage(page, wrapper);

        return employeeEntityPage.getRecords();
    }

    @Override
    public void batchUpdate(Collection<RdEmployeeEntity> employees) {
        if (CollectionUtils.isEmpty(employees)) {
            return;
        }

        try (SqlSession session = this.sqlSessionFactory.openSession(ExecutorType.BATCH)) {
            try {
                RdEmployeeMapper mapper = session.getMapper(RdEmployeeMapper.class);
                for (RdEmployeeEntity employee : employees) {
                    mapper.updateById(employee);
                }
                session.commit();
            } catch (Exception ex) {
                session.rollback();
                throw ex;
            }
        }
    }

    @Override
    public void resetExpiredStationIdByIds(Collection<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return;
        }

        this.employeeMapper.resetExpiredStationIdByIds(ids);
    }

    @Override
    public RdEmployeeEntity getByNoPkId(String id) {
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(RdEmployeeEntity::getId, id);

        return this.employeeMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<RdEmployeeEntity> getAllByNoPkIds(Collection<String> ids) {
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(RdEmployeeEntity::getId, ids);

        return this.employeeMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public Collection<String> getCheckingEmployeeIds(int pageIndex, int pageSize) {
        /*
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdEmployeeEntity::getStatus, EmploymentStatus.OffJob.ordinal());

        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit 0,%s", pageSize));
        lambdaQueryWrapper = lambdaQueryWrapper.select(RdEmployeeEntity::getId);

        List<Map<String, Object>> maps = this.employeeMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii -> String.valueOf(ii.get("id"))).collect(Collectors.toList());

         */
        if(pageIndex<1) {
            pageIndex = 1;
        }

        return this.tenantUserViewMapper.getCheckingEmployeeIds((pageIndex-1)* pageSize, pageSize);
    }

    @Override
    public RdEmployeeEntity getByTenantEmployeeId(String employeeId) {
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdEmployeeEntity::getTenantEmployeeId, employeeId);
        return this.employeeMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<String> getNonUsedRecords(int pageSize) {
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdEmployeeEntity::getSecurityStationId, BusinessConstants.DefaultEmptyValue);
        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("LIMIT %s", pageSize));

        lambdaQueryWrapper = lambdaQueryWrapper.select(RdEmployeeEntity::getId);
        List<Map<String, Object>> maps = this.employeeMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii -> String.valueOf(ii.get("id"))).collect(Collectors.toList());
    }

    @Override
    public int deleteNonUsedRecords(Collection<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return 0;
        }
        LambdaQueryWrapper<RdEmployeeEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(RdEmployeeEntity::getId, ids);
        /**
         * 临时不做严格的验证
         */
        //lambdaQueryWrapper = lambdaQueryWrapper.eq(RdEmployeeEntity::getSecurityStationId, BusinessConstants.DefaultEmptyValue);

        return this.employeeMapper.delete(lambdaQueryWrapper);
    }

    @Override
    public void batchUpdateLocations(Collection<UpdateEmployeeLocationDTO> data) {
        if (CollectionUtils.isEmpty(data)) {
            return;
        }

        this.employeeMapper.batchUpdateLocations(data);
    }
}
