package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.ProjectAdvanceSearchCriteria;
import com.bcxin.tenant.open.domains.entities.RdProjectEntity;
import com.bcxin.tenant.open.domains.entities.TenantEmployeeAttendanceEntity;
import com.bcxin.tenant.open.domains.mappers.RdProjectMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.enums.ProjectStatus;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

@Repository
public class RdProjectRepositoryImpl implements RdProjectRepository{
    private final RdProjectMapper projectMapper;

    public RdProjectRepositoryImpl(RdProjectMapper projectMapper) {
        this.projectMapper = projectMapper;
    }

    @Override
    public RdProjectEntity getById(Object id) {
        return this.projectMapper.selectById((Long)id);
    }

    @Override
    public void insert(RdProjectEntity data) {
        this.projectMapper.insert(data);
    }

    @Override
    public void update(RdProjectEntity data) {
        this.projectMapper.updateById(data);
    }

    @Override
    public Collection<RdProjectEntity> getByPage(Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdProjectEntity> wrapper = new LambdaQueryWrapper();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdProjectEntity::getId, ids);
        }

        IPage<RdProjectEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdProjectEntity> projectEntityPage =
                this.projectMapper.selectPage(page, wrapper);

        return projectEntityPage.getRecords();
    }

    @Override
    public EntityCollection<RdProjectEntity> search(ProjectAdvanceSearchCriteria criteria) {
        LambdaQueryWrapper<RdProjectEntity> lambdaQueryWrapper = new LambdaQueryWrapper();
        int pageIndex = criteria.getPageIndex();
        if (pageIndex <= 0) {
            pageIndex = 1;
        }

        if (StringUtils.hasLength(criteria.getName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.like(RdProjectEntity::getName, criteria.getName().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getType())) {
            lambdaQueryWrapper = lambdaQueryWrapper.eq(RdProjectEntity::getType, criteria.getType());
        }

        Date beginDate = criteria.getBeginDate();
        Date endDate = criteria.getEndDate();
        /*
        合并开始时间和结束时间
         */
        if (criteria.getStatus() != null) {
            var calendar = Calendar.getInstance();
            calendar.set(Calendar.HOUR, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            switch (criteria.getStatus()) {
                case Normal:
                    if (beginDate == null) {
                        beginDate = calendar.getTime();
                    } else if (calendar.getTime().after(beginDate)) {
                        beginDate = calendar.getTime();
                    }

                    //加一天进行计算
                    if (endDate == null) {
                        endDate = calendar.getTime();
                    } else if (calendar.getTime().before(endDate)) {
                        endDate = calendar.getTime();
                    }
                    break;
                case NotBegin:
                    if (beginDate == null) {
                        beginDate = calendar.getTime();
                    } else {
                        if (calendar.getTime().after(beginDate)) {
                            beginDate = calendar.getTime();
                        }
                    }
                    break;
                case EndYet:
                    if (endDate == null) {
                        endDate = calendar.getTime();
                    } else {
                        if (calendar.getTime().before(endDate)) {
                            endDate = calendar.getTime();
                        }
                    }
                    break;
            }
        }

        if (beginDate != null) {
            lambdaQueryWrapper = lambdaQueryWrapper.ge(RdProjectEntity::getBeginDate, beginDate);
        }

        if (endDate != null) {
            var endNewCalendar = Calendar.getInstance();
            endNewCalendar.setTime(endDate);
            endNewCalendar.add(1, Calendar.DAY_OF_MONTH);
            lambdaQueryWrapper = lambdaQueryWrapper.le(RdProjectEntity::getType, endNewCalendar.getTime());
        }

        IPage<RdProjectEntity> page = new Page<>(pageIndex, criteria.getPageSize());
        var projectEntityPage = this.projectMapper.selectPage(page, lambdaQueryWrapper);

        return EntityCollection.create(projectEntityPage.getRecords(), criteria.getPageSize(), projectEntityPage.getTotal());
    }

    @Override
    public RdProjectEntity getByV5Id(String id) {
        LambdaQueryWrapper<RdProjectEntity> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdProjectEntity::getId,id);
        return this.projectMapper.selectOne(lambdaQueryWrapper);
    }
}
