package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.ProjectTeamMemberCriteria;
import com.bcxin.tenant.open.domains.entities.RdProjectTeamMemberEntity;
import com.bcxin.tenant.open.domains.mappers.RdProjectTeamMemberMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.utils.StringUtil;
import org.springframework.stereotype.Repository;

@Repository
public class RdProjectTeamMemberRepositoryImpl implements RdProjectTeamMemberRepository{

    private final RdProjectTeamMemberMapper projectTeamGroupMapper;

    public RdProjectTeamMemberRepositoryImpl(RdProjectTeamMemberMapper projectTeamGroupMapper) {
        this.projectTeamGroupMapper = projectTeamGroupMapper;
    }


    @Override
    public RdProjectTeamMemberEntity getById(Object id) {
        return this.projectTeamGroupMapper.selectById((Long) id);
    }

    @Override
    public void insert(RdProjectTeamMemberEntity data) {
        this.projectTeamGroupMapper.insert(data);
    }

    @Override
    public void update(RdProjectTeamMemberEntity data) {
        this.projectTeamGroupMapper.updateById(data);
    }

    @Override
    public EntityCollection<RdProjectTeamMemberEntity> search(ProjectTeamMemberCriteria criteria) {
        LambdaQueryWrapper<RdProjectTeamMemberEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdProjectTeamMemberEntity::getProjectId, criteria.getProjectId());

        if (!StringUtil.isEmpty(criteria.getCompanyName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.like(RdProjectTeamMemberEntity::getCompanyName, criteria.getCompanyName().concat("%"));
        }

        if (!StringUtil.isEmpty(criteria.getPersonType())) {
            lambdaQueryWrapper = lambdaQueryWrapper.eq(RdProjectTeamMemberEntity::getPersonType, criteria.getPersonType());
        }

        if (!StringUtil.isEmpty(criteria.getIdCardNo())) {
            lambdaQueryWrapper = lambdaQueryWrapper.like(RdProjectTeamMemberEntity::getIdCardNo, criteria.getIdCardNo().concat("%"));
        }

        IPage<RdProjectTeamMemberEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

        var result = this.projectTeamGroupMapper.selectPage(page, lambdaQueryWrapper);

        return EntityCollection.create(result.getRecords(),(int)result.getSize(),result.getTotal());
    }
}
