package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.RdSecurityStationRailMessageCriteria;
import com.bcxin.tenant.open.domains.entities.RdCompanyEntity;
import com.bcxin.tenant.open.domains.entities.RdSecurityStationRailEntity;
import com.bcxin.tenant.open.domains.entities.RdSecurityStationRailMessageEntity;
import com.bcxin.tenant.open.domains.entities.TenantEmployeeAttendanceEntity;
import com.bcxin.tenant.open.domains.mappers.RdSecurityStationRailMessageMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.enums.RailBusinessType;
import com.bcxin.tenant.open.infrastructures.enums.ResourceType;
import com.bcxin.tenant.open.infrastructures.utils.DateUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.List;

@Repository
public class RdSecurityStationRailMessageRepositoryImpl implements RdSecurityStationRailMessageRepository{
    private final RdSecurityStationRailMessageMapper rdSecurityStationRailMessageMapper;

    public RdSecurityStationRailMessageRepositoryImpl(RdSecurityStationRailMessageMapper rdSecurityStationRailMessageMapper) {
        this.rdSecurityStationRailMessageMapper = rdSecurityStationRailMessageMapper;
    }

    @Override
    public RdSecurityStationRailMessageEntity getById(Object id) {
        RdSecurityStationRailMessageEntity message = this.rdSecurityStationRailMessageMapper.selectById((Serializable) id);

        return message;
    }

    @Override
    public void insert(RdSecurityStationRailMessageEntity data) {
        this.rdSecurityStationRailMessageMapper.insert(data);
    }

    @Override
    public void update(RdSecurityStationRailMessageEntity data) {
        this.rdSecurityStationRailMessageMapper.updateById(data);
    }

    @Override
    public void insertAll(Collection<RdSecurityStationRailMessageEntity> messages) {
        if (CollectionUtils.isEmpty(messages)) {
            return;
        }

        messages.forEach(mg -> {
            this.rdSecurityStationRailMessageMapper.insert(mg);
        });
    }

    @Override
    public EntityCollection<RdSecurityStationRailMessageEntity> search(RdSecurityStationRailMessageCriteria criteria) {
        LambdaQueryWrapper<RdSecurityStationRailMessageEntity> lambdaQueryWrapper
                = new LambdaQueryWrapper<>();

        /**
         * 当getCurrentSuperviseDepartIds不为空时为监管调度台，否则为企业调度台
         */
        if (!CollectionUtils.isEmpty(criteria.getCurrentSuperviseDepartIds())) {
            lambdaQueryWrapper = lambdaQueryWrapper.in(RdSecurityStationRailMessageEntity::getRailReferenceSuperviseDepartId, criteria.getCurrentSuperviseDepartIds());
        } else {
            lambdaQueryWrapper = lambdaQueryWrapper.in(RdSecurityStationRailMessageEntity::getOrganizationId, criteria.getOrganizationIds());
        }
        if (StringUtils.hasLength(criteria.getEmployeeName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailMessageEntity::getEmployeeName, criteria.getEmployeeName());
        }
        if (StringUtils.hasLength(criteria.getCompanyName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailMessageEntity::getCompanyName, criteria.getCompanyName());
        }
        if (StringUtils.hasLength(criteria.getRailReferenceName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailMessageEntity::getRailReferenceName, criteria.getRailReferenceName());
        }
        if (!CollectionUtils.isEmpty(criteria.getSuperviseDepartIds())) {
            lambdaQueryWrapper =
                    lambdaQueryWrapper.in(RdSecurityStationRailMessageEntity::getRailReferenceSuperviseDepartId, criteria.getSuperviseDepartIds());
        }

        if (criteria.getEventStartDate() != null) {
            lambdaQueryWrapper = lambdaQueryWrapper.ge(RdSecurityStationRailMessageEntity::getEventTime, criteria.getEventStartDate());
        }
        if (criteria.getEventEndDate() != null) {
            Timestamp endTime = DateUtils.getSpecialDayEndTimestamp(criteria.getEventEndDate());
            lambdaQueryWrapper = lambdaQueryWrapper.le(RdSecurityStationRailMessageEntity::getEventTime, endTime);
        }
        if (StringUtils.hasLength(criteria.getEmployeeIdCardNo())) {
            lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailMessageEntity::getEmployeeIdCardNo, criteria.getEmployeeIdCardNo());
        }
        if (criteria.getDeskTypes() != null) {
            int calculatedResourceTypeValue = ResourceType.getCalculatedResourceValueByDeskTypes(criteria.getDeskTypes());
            lambdaQueryWrapper = lambdaQueryWrapper.apply("calculated_resource_type&" + calculatedResourceTypeValue + ">0");
        }

        if (StringUtils.hasLength(criteria.getEmployeeName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailMessageEntity::getEmployeeName, criteria.getEmployeeName());
        }

        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(RdSecurityStationRailMessageEntity::getCreatedTime);

        if (!criteria.ignorePage()) {
            IPage<RdSecurityStationRailMessageEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

            IPage<RdSecurityStationRailMessageEntity> securityStationRailMessageEntityIPage =
                    this.rdSecurityStationRailMessageMapper.selectPage(page, lambdaQueryWrapper);

            return EntityCollection.create(
                    securityStationRailMessageEntityIPage.getRecords(), criteria.getPageSize(),
                    securityStationRailMessageEntityIPage.getTotal());
        } else {

            /**
             * 按照分页取出数据
             */
            lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit %s, %s ", criteria.getSkip(), criteria.getPageSize()));
            Collection<RdSecurityStationRailMessageEntity> securityStationRailMessageEntityIPage
                    = this.rdSecurityStationRailMessageMapper.selectList(lambdaQueryWrapper);

            return EntityCollection.createForNoTotalCount(securityStationRailMessageEntityIPage, criteria.getPageSize());
        }
    }
}
