package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.RdSecurityStationRailCriteria;
import com.bcxin.tenant.open.domains.entities.DeviceCommunicatedLogEntity;
import com.bcxin.tenant.open.domains.entities.RdSecurityStationRailEntity;
import com.bcxin.tenant.open.domains.mappers.RdSecurityStationRailMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.TenantContext;
import com.bcxin.tenant.open.infrastructures.TenantEmployeeContext;
import com.bcxin.tenant.open.infrastructures.enums.RailBusinessType;
import com.bcxin.tenant.open.infrastructures.exceptions.UnAuthorizedTenantException;
import com.bcxin.tenant.open.infrastructures.utils.DateUtils;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.List;

@Repository
public class RdSecurityStationRailRepositoryImpl implements RdSecurityStationRailRepository {

    private final RdSecurityStationRailMapper securityStationRailMapper;

    public RdSecurityStationRailRepositoryImpl(RdSecurityStationRailMapper securityStationRailMapper) {
        this.securityStationRailMapper = securityStationRailMapper;
    }


    @Override
    public RdSecurityStationRailEntity getById(Object id) {
        return this.securityStationRailMapper.selectById((long) id);
    }

    @Override
    public void insert(RdSecurityStationRailEntity data) {
        this.securityStationRailMapper.insert(data);
    }

    @Override
    public void update(RdSecurityStationRailEntity data) {
        this.securityStationRailMapper.updateById(data);
    }

    @Override
    public Collection<RdSecurityStationRailEntity> search(RdSecurityStationRailCriteria criteria) {
        LambdaQueryWrapper<RdSecurityStationRailEntity> lambdaQueryWrapper
                = new LambdaQueryWrapper<>();
        TenantEmployeeContext.TenantUserModel userModel = TenantContext.getInstance().getUserContext().get();
        if (userModel == null) {
            throw new UnAuthorizedTenantException();
        }

        lambdaQueryWrapper = lambdaQueryWrapper
                .eq(RdSecurityStationRailEntity::isDeleted, false);

        if (!userModel.isSuperviseDepartRole()) {
            lambdaQueryWrapper =
                    lambdaQueryWrapper.in(RdSecurityStationRailEntity::getOrganizationId, userModel.getNoSuperviseScopePermissions());
        } else if (criteria.getSearchModel() != RdSecurityStationRailCriteria.SearchModel.SecurityResource) {
            lambdaQueryWrapper =
                    lambdaQueryWrapper.in(RdSecurityStationRailEntity::getSuperviseDepartId, userModel.getAssignedSuperviseDepartIds());
        }

        switch (criteria.getSearchModel()) {
            case Search -> {
                if (StringUtils.hasLength(criteria.getKeyWord())) {
                    lambdaQueryWrapper = lambdaQueryWrapper.likeRight(RdSecurityStationRailEntity::getName, criteria.getKeyWord());
                }
            }
            case SecurityResource -> {
                Collection<String> selectedDepartIds = userModel.getMatchedSuperviseDepartIds(criteria.getSuperviseDepartIds());
                if (!CollectionUtils.isEmpty(selectedDepartIds)) {
                    lambdaQueryWrapper = lambdaQueryWrapper.in(RdSecurityStationRailEntity::getSuperviseDepartId, selectedDepartIds);
                }
            }
            case Station -> {
                if (StringUtils.hasLength(criteria.getStationId())) {
                    lambdaQueryWrapper = lambdaQueryWrapper
                            .eq(RdSecurityStationRailEntity::getReferenceNumber, criteria.getStationId())
                            .eq(RdSecurityStationRailEntity::getBusinessType, RailBusinessType.Station.ordinal());
                }
            }
        }

        lambdaQueryWrapper = lambdaQueryWrapper.orderByDesc(RdSecurityStationRailEntity::getLastUpdatedTime);

        List<RdSecurityStationRailEntity> data = this.securityStationRailMapper.selectList(lambdaQueryWrapper);

        return data;
    }

    @Override
    public Collection<RdSecurityStationRailEntity> getValidRails(Collection<String> stationIds) {
        LambdaQueryWrapper<RdSecurityStationRailEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.in(RdSecurityStationRailEntity::getReferenceNumber, stationIds)
                .eq(RdSecurityStationRailEntity::getBusinessType, RailBusinessType.Station);
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RdSecurityStationRailEntity::isDeleted, false);
        lambdaQueryWrapper = lambdaQueryWrapper.le(RdSecurityStationRailEntity::getBeginTime, DateUtils.getToday());
        lambdaQueryWrapper = lambdaQueryWrapper.ge(RdSecurityStationRailEntity::getEndTime, DateUtils.getToday());

        return this.securityStationRailMapper.selectList(lambdaQueryWrapper);
    }
}
