package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.entities.RdSecurityStationEntity;
import com.bcxin.tenant.open.domains.mappers.RdSecurityStationMapper;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import java.util.*;
import java.util.stream.Collectors;

@Repository
public class RdSecurityStationRepositoryImpl implements RdSecurityStationRepository {
    private final RdSecurityStationMapper rdSecurityStationMapper;

    public RdSecurityStationRepositoryImpl(RdSecurityStationMapper rdSecurityStationMapper) {
        this.rdSecurityStationMapper = rdSecurityStationMapper;
    }

    @Override
    public RdSecurityStationEntity getById(Object pkId) {
        long id = (long) pkId;
        RdSecurityStationEntity stationEntity =
                this.rdSecurityStationMapper.selectById(id);

        return stationEntity;
    }

    @Override
    public void insert(RdSecurityStationEntity data) {
        this.rdSecurityStationMapper.insert(data);
    }

    @Override
    public void update(RdSecurityStationEntity data) {
        this.rdSecurityStationMapper.updateById(data);
    }

    @Override
    public Collection<RdSecurityStationEntity> getByPage(Collection<String> ids, int pageIndex, int pageSize) {
        LambdaQueryWrapper<RdSecurityStationEntity> wrapper = new LambdaQueryWrapper();
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdSecurityStationEntity::getId, ids);
        }

        if (pageIndex <= 0) {
            pageIndex = 1;
        }

        IPage<RdSecurityStationEntity> page = new Page<>(pageIndex, pageSize);

        IPage<RdSecurityStationEntity> rdCompanyEntityPage =
                this.rdSecurityStationMapper.selectPage(page, wrapper);

        return rdCompanyEntityPage.getRecords();
    }

    @Override
    public void updateCountOfSecurityMen(Collection<String> stationIds) {
        this.rdSecurityStationMapper.updateCountOfSecurityMen(stationIds);
    }

    @Override
    public RdSecurityStationEntity getByNoPkId(String id) {
        LambdaQueryWrapper<RdSecurityStationEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(RdSecurityStationEntity::getId, id);

        return this.rdSecurityStationMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Collection<RdSecurityStationEntity> getByIds(Collection<String> ids) {
        LambdaQueryWrapper<RdSecurityStationEntity> wrapper = new LambdaQueryWrapper();
        if (!CollectionUtils.isEmpty(ids)) {
            wrapper = wrapper.in(RdSecurityStationEntity::getId, ids);
        }

        return this.rdSecurityStationMapper.selectList(wrapper);
    }

    @Override
    public Collection<String> getInvalidStationIds(int pageSize) {
        LambdaQueryWrapper<RdSecurityStationEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.notIn(RdSecurityStationEntity::getSiteState, "已完善", "生效");

        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        lambdaQueryWrapper = lambdaQueryWrapper.lt(RdSecurityStationEntity::getEndDate, calendar.getTime());

        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit 0,%s", pageSize));
        lambdaQueryWrapper = lambdaQueryWrapper.select(RdSecurityStationEntity::getId);

        List<Map<String, Object>> maps = this.rdSecurityStationMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii->String.valueOf(ii.get("id"))).collect(Collectors.toList());
    }
}
