package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.RollCallPlanSearchCriteria;
import com.bcxin.tenant.open.domains.entities.RollCallPlanEntity;
import com.bcxin.tenant.open.domains.mappers.RollCallPlanMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.enums.RepetitiveRule;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.stream.Collectors;

@Component
public class RollCallPlanRepositoryImpl implements RollCallPlanRepository {
    private final RollCallPlanMapper rollCallPlanMapper;

    public RollCallPlanRepositoryImpl(RollCallPlanMapper rollCallPlanMapper) {
        this.rollCallPlanMapper = rollCallPlanMapper;
    }

    @Override
    public RollCallPlanEntity getById(Object id) {
        return this.rollCallPlanMapper.selectById((Long) id);
    }

    @Override
    public void insert(RollCallPlanEntity data) {
        this.rollCallPlanMapper.insert(data);
    }

    @Override
    public void update(RollCallPlanEntity data) {
        this.rollCallPlanMapper.updateById(data);
    }

    @Override
    public EntityCollection<RollCallPlanEntity> search(RollCallPlanSearchCriteria criteria) {
        LambdaQueryWrapper<RollCallPlanEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallPlanEntity::isDeleted, false);
        if (StringUtils.hasLength(criteria.getName())) {
            lambdaQueryWrapper = lambdaQueryWrapper.like(RollCallPlanEntity::getName, "%".concat(criteria.getName()).concat("%"));
        }

        if (!CollectionUtils.isEmpty(criteria.getSuperviseDepartIds())) {
            lambdaQueryWrapper = lambdaQueryWrapper.in(RollCallPlanEntity::getSuperviseDepartId, criteria.getSuperviseDepartIds());
        }

        /**
         * 采用json的方式进行数据的过滤
         */
        if (!CollectionUtils.isEmpty(criteria.getStationTypes())) {
            String filterValue =
                    criteria.getStationTypes().stream().map(ii -> String.format("\"%s\"", ii)).distinct().collect(Collectors.joining(","));

            lambdaQueryWrapper = lambdaQueryWrapper.apply("JSON_CONTAINS(station_types,'[" + filterValue + "]')");
        }

        IPage<RollCallPlanEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

        IPage<RollCallPlanEntity> rollCallPlanEntityIPage = this.rollCallPlanMapper.selectPage(page, lambdaQueryWrapper);

        return EntityCollection.create(rollCallPlanEntityIPage.getRecords(), criteria.getPageSize(), rollCallPlanEntityIPage.getTotal());
    }

    @Override
    public Collection<RollCallPlanEntity> getScheduledRollCallPlans(Timestamp scheduledTime, RepetitiveRule repetitiveRule) {
        LambdaQueryWrapper<RollCallPlanEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RollCallPlanEntity::isDeleted, false);

        Calendar scheduledDate = Calendar.getInstance();
        scheduledDate.setTime(scheduledTime);
        scheduledDate.set(Calendar.HOUR_OF_DAY, 0);
        scheduledDate.set(Calendar.MINUTE, 0);
        scheduledDate.set(Calendar.SECOND, 0);

        lambdaQueryWrapper = lambdaQueryWrapper.le(RollCallPlanEntity::getBeginDate, scheduledDate.getTime());

        Calendar endDateCalendar = Calendar.getInstance();
        endDateCalendar.setTime(scheduledDate.getTime());
        endDateCalendar.add(Calendar.DAY_OF_MONTH, 1);
        Date endScheduledDate = endDateCalendar.getTime();

        lambdaQueryWrapper = lambdaQueryWrapper.gt(RollCallPlanEntity::getEndDate, endScheduledDate);

        lambdaQueryWrapper = lambdaQueryWrapper.apply("repetitive_rule & " + repetitiveRule.getValue() + " > 0");

        /*
        先排除时间点的过滤
        SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
        String scheduledTimeValue = dateFormat.format(scheduledTime);

        lambdaQueryWrapper = lambdaQueryWrapper.apply("periods_of_times->'$.value'>='" + scheduledTimeValue + "'");
         */
        Collection<RollCallPlanEntity> result = this.rollCallPlanMapper.selectList(lambdaQueryWrapper);

        return result;
    }
}
