package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.bcxin.tenant.open.domains.entities.RoomEntity;
import com.bcxin.tenant.open.domains.entities.RoomUserEntity;
import com.bcxin.tenant.open.domains.mappers.RoomMapper;
import com.bcxin.tenant.open.domains.mappers.RoomUserMapper;
import com.bcxin.tenant.open.infrastructures.enums.RoomUserStatus;
import com.bcxin.tenant.open.infrastructures.enums.ProcessedStatus;
import org.springframework.stereotype.Repository;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Repository
public class RoomRepositoryImpl implements RoomRepository {
    private final RoomMapper roomMapper;
    private final RoomUserMapper roomUserMapper;

    public RoomRepositoryImpl(RoomMapper roomMapper, RoomUserMapper roomUserMapper) {
        this.roomMapper = roomMapper;
        this.roomUserMapper = roomUserMapper;
    }

    @Override
    public RoomEntity getById(Object id) {
        return null;
    }

    @Override
    public void update(RoomEntity data) {

    }

    @Override
    public void insert(RoomEntity room) {
        this.roomMapper.customInsert(room);
    }

    @Override
    public Collection<Long> getReadyForCalculatedRoomIds(int pageSize) {
        LambdaQueryWrapper<RoomEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RoomEntity::getTencentClosed, true);
        lambdaQueryWrapper = lambdaQueryWrapper.eq(RoomEntity::getStatus, ProcessedStatus.Init.ordinal());
        lambdaQueryWrapper = lambdaQueryWrapper.last(String.format("limit 0,%s", pageSize));
        lambdaQueryWrapper = lambdaQueryWrapper.select(RoomEntity::getId);

        List<Map<String, Object>> maps = this.roomMapper.selectMaps(lambdaQueryWrapper);

        return maps.stream().map(ii -> (Long) ii.get("id")).collect(Collectors.toList());
    }

    @Override
    public void addRoomUsers(Collection<RoomUserEntity> users) {
        this.roomMapper.addUsers(users);
    }

    @Override
    public Collection<RoomUserEntity> getRoomUsersByRoomIds(Collection<String> roomIds) {
        LambdaQueryWrapper<RoomUserEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper = lambdaQueryWrapper.in(RoomUserEntity::getRoomId, roomIds);

        return this.roomUserMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public void removeUsers(long roomId, Collection<String> tencentUserIds) {
        LambdaUpdateWrapper<RoomUserEntity> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper = lambdaUpdateWrapper.set(RoomUserEntity::getStatus, RoomUserStatus.REMOVE.ordinal())
                .set(RoomUserEntity::getLeaveTime, Timestamp.from(Instant.now()))
                .eq(RoomUserEntity::getRoomId,roomId)
                .in(RoomUserEntity::getTencentUserId,tencentUserIds);
        this.roomUserMapper.update(null,lambdaUpdateWrapper);
    }
}
