package com.bcxin.tenant.open.domains.repositories;

import com.bcxin.tenant.open.domains.dtos.ProcessTencentCallbackLogDTO;
import com.bcxin.tenant.open.domains.entities.TencentCallbackLogEntity;
import com.bcxin.tenant.open.domains.mappers.TencentCallbackLogMapper;
import com.bcxin.tenant.open.domains.views.TencentCallbackLogView;
import com.bcxin.tenant.open.infrastructures.exceptions.NotSupportTenantException;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

@Repository
public class TencentCallbackLogRepositoryImpl implements TencentCallbackLogRepository {
    private final TencentCallbackLogMapper tencentCallbackLogMapper;
    private final SqlSessionFactory sessionFactory;

    public TencentCallbackLogRepositoryImpl(TencentCallbackLogMapper tencentCallbackLogMapper,
                                            SqlSessionFactory sessionFactory) {
        this.tencentCallbackLogMapper = tencentCallbackLogMapper;
        this.sessionFactory = sessionFactory;
    }

    @Override
    public TencentCallbackLogEntity getById(Object id) {
        return this.tencentCallbackLogMapper.selectById((Long) id);
    }

    @Override
    public void insert(TencentCallbackLogEntity data) {
        this.tencentCallbackLogMapper.customInsert(data);
    }

    @Override
    public void update(TencentCallbackLogEntity data) {
        throw new NotSupportTenantException("不支持该操作");
    }

    /**
     * https://mybatis.org/mybatis-dynamic-sql/docs/insert.html
     * https://zhuanlan.zhihu.com/p/434741532#:~:text=Insert%20inside%20Mybatis%20foreach%20is%20not%20batch%2C%20this,error%20if%20the%20statement%20itself%20become%20too%20large.
     * https://blog.csdn.net/aijian3049/article/details/101144369
     *
     * @param logs
     */
    @Override
    public void batchInsert(Collection<TencentCallbackLogEntity> logs) {
        if (CollectionUtils.isEmpty(logs)) {
            return;
        }

        try (SqlSession session = sessionFactory.openSession(ExecutorType.BATCH, false)) {
            TencentCallbackLogMapper mapper = session.getMapper(TencentCallbackLogMapper.class);
            for (TencentCallbackLogEntity log : logs) {
                mapper.customInsert(log);
            }

            session.commit();
        }
    }

    @Override
    public Collection<String> getTopExitRoomButPendingIds(Collection<Long> exitRoomLogIds) {
        Collection<String> roomIds = this.tencentCallbackLogMapper.getTopExitRoomButPendingIds(exitRoomLogIds);
        if (CollectionUtils.isEmpty(roomIds)) {
            return Collections.EMPTY_LIST;
        }

        return roomIds.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public Collection<TencentCallbackLogView> getAllPendingLogsByRoomIds(Collection<Long> roomIds) {
        Collection<TencentCallbackLogView>
                callbackLogViews = this.tencentCallbackLogMapper.getAllPendingLogsByRoomIds(roomIds);

        return callbackLogViews;
    }

    @Override
    public void process(ProcessTencentCallbackLogDTO callbackLogDTO) {
        this.tencentCallbackLogMapper.process(callbackLogDTO);
    }
}
